import tkinter as tk
from math import sin, cos
from PIL import ImageTk, Image
try:
    from scipy.constants import g, pi
except (ModuleNotFoundError, ImportError):
    g = 9.80665
    pi = 3.141592653589793


WIDTH = 1300
HEIGHT = 650
ARR = []
DETAILS = True
INFO = []
FORCES = []
X, Y = 20, 580


def get():
    return s1.get() * pi / 180, s2.get() / 3.6


def forces(*args):
    global FORCES
    angle, power = get()
    delete(FORCES)
    FORCES = [canvas.create_line(X, Y, X + power * cos(angle), Y - power * sin(angle),
                                 arrow=tk.LAST),
              canvas.create_arc(X - 30, Y - 30, X + 30, Y + 30, start=0,
                                extent=angle * 180 / pi, style=tk.ARC),
              canvas.create_text(X + 40, Y, text=f"a = {round(angle * 180 / pi, 1)}°",
                                 anchor="sw", font=("Roboto", 15, "italic"))]


def trajectory():
    angle, power = get()
    time = 2 * power * sin(angle) / g + 10
    step = 0
    pos = [(X, Y)]
    while step <= time:
        x1 = X + power * step * cos(angle)
        y1 = Y - power * step * sin(angle) + 0.5 * g * pow(step, 2)
        ARR.append(canvas.create_line(x1, y1, pos[-1]))
        pos.append((x1, y1))
        step += 0.1

    if DETAILS:
        info(angle, power)


def delete(arr):
    for i in arr:
        canvas.delete(i)


def delete_but():
    global ARR
    delete(ARR)
    ARR = []


def details():
    global DETAILS, INFO
    DETAILS = not DETAILS
    if DETAILS:
        b5.configure(bg="limegreen")
    else:
        b5.configure(bg="SystemButtonFace")
        delete(INFO)
        INFO = []


def info(angle, power):
    global INFO
    delete(INFO)
    INFO = [canvas.create_text(1230, 111,
                               text=f"Equation of trajectory\n"
                                    f"Distance: {round((power ** 2) / g * sin(2 * angle), 3)}m\n"
                                    f"Max height: {round((power ** 2) * (sin(angle) ** 2) / (2 * g), 3)}m",
                               anchor="ne",
                               font=("Roboto", "20")),
            canvas.create_image(1230, 20, image=eq, anchor="ne")]


root = tk.Tk()
root.title("trajectory")


canvas = tk.Canvas(root, width=WIDTH, height=HEIGHT-50, bg="white")
canvas.grid(row=0, column=1, columnspan=5)
canvas.create_line(20, 0, 20, 600, fill="grey")
canvas.create_line(0, 580, 1300, 580, fill="grey")

eq = ImageTk.PhotoImage(Image.open("rovnica.png"))

s1 = tk.Scale(root, from_=90, to=30, resolution=0.2, command=forces)
s1.grid(row=0, column=0, sticky="nesw")

s2 = tk.Scale(root, from_=300, to=400, orient="horizontal", command=forces)
s2.grid(row=1, column=5, sticky="nesw")

b1 = tk.Button(root, text="Fire", command=trajectory)
b1.grid(row=1, column=0, sticky="nesw")

b2 = tk.Button(root, text="Delete lines", command=delete_but)
b2.grid(row=1, column=1, sticky="nesw")

b3 = tk.Button(root, text="Max distance", command=lambda: s1.set(45) or trajectory())
b3.grid(row=1, column=2, sticky="nesw")

b4 = tk.Button(root, text="Max height", command=lambda: s1.set(90) or trajectory())
b4.grid(row=1, column=3, sticky="nesw")

b5 = tk.Button(root, text="Details", command=details, bg="limegreen")
b5.grid(row=1, column=4, sticky="nesw")

root.mainloop()
